<?php

namespace Drupal\commerce_comgate\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Comgate\Request\CreatePayment;

/**
 * Provides the class for payment off-site form.
 *
 * Provide a buildConfigurationForm() method which calls buildRedirectForm()
 * with the right parameters.
 */
class ComgateRedirectForm extends BasePaymentOffsiteForm {

  /**
   * Gateway plugin.
   *
   * @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface
   */
  private $paymentGatewayPlugin;

  /**
   * Getting plugin's configuration.
   *
   * @param string $configuration
   *   Configuration name.
   *
   * @return mixed
   *   Configuration value.
   */
  private function getConfiguration($configuration) {
    return $this->paymentGatewayPlugin->getConfiguration()[$configuration] ?? NULL;
  }

  /**
   * Create URL.
   *
   * @param string $route
   *   The route name.
   *
   * @return \Drupal\Core\GeneratedUrl|string
   *   Url.
   */
  public function url(string $route) {
    return Url::fromRoute($route, [], ['absolute' => TRUE])->toString();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $this->paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();
    $order = $payment->getOrder();
    $redirect_url = $this->getConfiguration('comgate_gate_url') . 'create';
    $order_id = $payment->getOrderId();
    $currency_id = $this->getConfiguration('comgate_currency');
    $country = $this->getConfiguration('comgate_country');
    /** @var \Drupal\user\Entity\User $user */
    $user_email = '';
    if ($order_mail = $order->getEmail()) {
      $user_email = $order_mail;
    }
    $total_price_to_comgate = (int)round($order->getTotalPrice()->getNumber() * 100);
    $payment_msg = 'Order #'.$order->id();
    $createPayment = new CreatePayment($total_price_to_comgate, $order_id, $user_email, $payment_msg);
    $createPayment->setCountry($country)->setCurr($currency_id);
    if ($method = $this->getConfiguration('comgate_method')) {
      $createPayment->setMethod($method);
    }
    $data = $createPayment->getData();
    $data += [
      'merchant' => $this->getConfiguration('comgate_merchant_id'),
      'test' => ($this->getConfiguration('mode') == 'sandbox') ? 'true' : 'false',
    ];
    return $this->buildRedirectForm($form, $form_state, $redirect_url, $data, self::REDIRECT_POST);
  }

}
